
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

import java.util.Vector;

public class ASCIIDataFileParserEngine implements ParserEngine
{

  public ASCIIDataFileParserEngine()
  {
    psMachine = new ASCIIDataFileParserStateMachine();
    recordContainer = new Vector();
  }

  public void setFieldDelimiter(byte fieldDelim)
  {
    psMachine.setFieldDelim(fieldDelim);
  }

  public void parseBuffer(byte[] buff, int len) throws ParserException
  {
    String f1,f2,f3,f4,f5,f6;
    CustomerRec cRec;
    int f;

    f1 = f2 = f3 = f4 = f5 = f6 = null;
    psMachine.resetParser();
    f = 1;

    for (int i = 0; i < len; i++)
    {
      try
      {
        psMachine.processByte(buff[i]);
      } catch (ParserException e) { throw e; }

      if (psMachine.fieldDetected() == true)
      {
        // A new field delimiter was detected. Store the data as a field
        // and assign it to the right customer record field
        if (f == 1)
        {
          f1 = psMachine.getData();
          f++;
        }
        else if (f == 2)
        {
          f2 = psMachine.getData();
          f++;
        }
        else if (f == 3)
        {
          f3 = psMachine.getData();
          f++;
        }
        else if (f == 4)
        {
          f4 = psMachine.getData();
          f++;
        }
        else if (f == 5)
        {
          f5 = psMachine.getData();
          f++;
        }
        psMachine.resetDataBuffer();
      }
      if (psMachine.EORDetected() == true)
      {
        if (f <= 5)
        {
          // We have fewer then anticipated fields
          ParserException e = new
            ParserException(f1+","+f2+","+f3+","+f4+","+f5+
                                    " Missing field detected");
          psMachine.resetDataBuffer();
          f = 1;
          throw e;
        }
        else
        {
          // Get the last field
          f6 = psMachine.getData();
          cRec = new CustomerRec(f1,f2,f3,f4,f5,f6);
          recordContainer.add(cRec);
          psMachine.resetDataBuffer();
          f = 1;
        }
      }
    }
  }

  public Vector getRecordContainer()
  {
    return(recordContainer);
  }

  private ASCIIDataFileParserStateMachine psMachine;
  private Vector recordContainer; // Each record in this container is an
                                  // instance of CustomerRec
}