
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

public class ASCIIDataFileParserStateMachine
{

  public ASCIIDataFileParserStateMachine()
  {
    resetParser();
  }

  public boolean fieldDetected()
  {
    if (currentState == endOfField)
      return(true);
    return(false);
  }

  public boolean EORDetected()
  {
    if (currentState == endOfRecord)
      return(true);
    return(false);
  }

  public void resetParser()
  {
    currentState = idleState;
    dataIterator = 0;
    parsedData = new byte[10240+1];
  }

  public String getData()
  {
    String s;
    if (dataIterator == 0)
      return((String)null); // No data has been parsed or the statemachine
                            // has been reset.
    s = new String(parsedData,0,dataIterator);
    resetDataBuffer();
    return(s);
  }

  public void resetDataBuffer()
  {
    dataIterator = 0;
  }

  public void processByte(byte a) throws ParserException {
    switch (currentState)
    {
      case idleState:
        if (a == fieldDelim) // Start of a element tag is detected
        {
          currentState = endOfField; // Change state
          break;
        }
        if (a == EOR)
        {
          currentState = endOfRecord;
        }
        else
        {
          // Store into parsedData
          parsedData[dataIterator] = a;
          dataIterator++;
          if (dataIterator > 10240)
          {
            ParserException e = new ParserException("Data field too long");
            throw e;
          }
        }
      break;

      case endOfField:
        if (a == EOR) // Tag end delimiter detected
        {
          currentState = endOfRecord; // End of tag is detected
          dataIterator = 0;
          break;
        }
        else
        {
          // Store data in parsedData
          parsedData[dataIterator] = a;
          dataIterator++;
          currentState = idleState;
          if (dataIterator > 10240)
          {
            ParserException e = new ParserException("Data field too long");
            throw e;
          }
        }
      break;

      case endOfRecord:
        parsedData[dataIterator] = a;
        dataIterator++;
        currentState = idleState;
        if (dataIterator > 10240)
        {
          ParserException e = new ParserException("Data field too long");
          throw e;
        }
      break;
    }
    return;
  }

  public void setFieldDelim(byte a)
  {
    fieldDelim = a;
  }
  public byte getFieldDelim()
  {
    return(fieldDelim);
  }

  // private member variables

  private static final int idleState = 0;
  private static final int endOfField = 1;
  private static final int endOfRecord = 2;
  private static final byte EOR = 13;
  private byte fieldDelim;
  private byte[] parsedData;
  private int dataIterator;
  private int currentState;
}