
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

import java.util.Vector;
import java.io.IOException;

public class CustomerDataServiceAPIImpl implements CustomerDataServiceAPI
{

  public CustomerDataServiceAPIImpl()
  {
    customerRecs = null;
    metadataRecs = null;
  }

  public void loadCustomerRecords(String metaDataFileName, String dataFileName)
                            throws Exception, IOException, ParserException
  {
    ASCIIDataFileParser dfp;
    TagValueParser tvp;
    TagValueParserEngine tvpEngine;

    int sFlag;

    // Create an instance of tag value parser engine. We only do this so that
    // we can assign our own meta data record container and pass it to the
    // engine.
    tvpEngine = new TagValueParserEngine();
    // Creat a new instance of MetadataRec
    metadataRecs = new MetadataRec();
    // Assign it to the tag value parser engine
    tvpEngine.setMetadataRec(metadataRecs);
    // Associate the engine with the tag value parser
    tvp = new TagValueParser(tvpEngine);

    // Create an instance of ASCII file parser
    dfp = new ASCIIDataFileParser();
    customerRecs = dfp.getRecordContainer(); // We can extract
                                             // individual records later

    sFlag = 0;

    try
    {
      // Since the meta data definition file is in a tag value format
      // The tag value parser is used to parse the file and store the
      // the data in Java objects
      tvp.processFile(metaDataFileName);
    }
    catch (Exception e)
    {
      // Exceptions thrown indicating errors in the parsing cycles
      sFlag = 1;
      throw e;
    }

    if (sFlag == 0)
    {
      // No errors during parsing the meta data definition file

      // Retrieve the control fields FIELD_DELIM and DEFAULT_DATAFILENAME
      // The FIELD_DELIM record indicates the field delimiter used in the
      // customer data file and the DEFAULT_DATAFILENAME record indicates
      // the name of the default customer data file name
      String s = tvp.getControlRecord("FIELD_DELIM");
      byte [] b = s.getBytes();
      // Set the appropriate field delimiter
      dfp.setFieldDelimiter(b[0]);

      try
      {
        // Parse the customer data file using the ASCII data file parser
        dfp.processFile(dataFileName);
      }
      catch (Exception e)
      {
        throw e;
      }
    }
    return;
  }

  public void loadCustomerRecords(String metaDataFileName) throws Exception,
                            ParserException, IOException
  {
    ASCIIDataFileParser dfp;
    TagValueParser tvp;
    TagValueParserEngine tvpEngine;

    int sFlag;

    // Create an instance of tag value parser engine. We only do this so that
    // we can assign our own meta data record container and pass it to the
    // engine.
    tvpEngine = new TagValueParserEngine();
    // Creat a new instance of MetadataRec
    metadataRecs = new MetadataRec();
    // Assign it to the tag value parser engine
    tvpEngine.setMetadataRec(metadataRecs);
    // Associate the engine with the tag value parser
    tvp = new TagValueParser(tvpEngine);

    // Create an instance of ASCII file parser
    dfp = new ASCIIDataFileParser();
    customerRecs = dfp.getRecordContainer(); // We can extract
                                             // individual records later

    sFlag = 0;

    try
    {
      // Since the meta data definition file is in a tag value format
      // The tag value parser is used to parse the file and store the
      // the data in Java objects
      tvp.processFile(metaDataFileName);
    }
    catch(Exception e)
    {
      // Exceptions thrown indicating errors in the parsing cycles
      sFlag = 1;
      throw e;
    }

    if (sFlag == 0)
    {
      // No errors during parsing the meta data definition file

      // Retrieve the control fields FIELD_DELIM and DEFAULT_DATAFILENAME
      // The FIELD_DELIM record indicates the field delimiter used in the
      // customer data file and the DEFAULT_DATAFILENAME record indicates
      // the name of the default customer data file name
      String s = tvp.getControlRecord("FIELD_DELIM");
      byte [] b = s.getBytes();
      // Set the appropriate field delimiter
      dfp.setFieldDelimiter(b[0]);

      String f = tvp.getControlRecord("DEFAULT_DATAFILENAME");
      try
      {
        // Parse the customer data file using the ASCII data file parser
        dfp.processFile(f);
      }
      catch(Exception e)
      {
        throw e;
      }
    }
    return;
  }

  public boolean isCustomerRecordValid(CustomerRec aRec)
  {
    if (aRec.getID() == null)
      return(false);
    if (aRec.getName() == null)
      return(false);
    if (aRec.getRecAction() == 0x00)
      return(false);
    if (aRec.getAddress() == null)
      return(false);
    if (aRec.getStatus() == null)
      return(false);
    if (aRec.getOutstandingBal() < (float)0.00)
      return(false);

    return(true);
  }

  public Vector getCustomerRecordTable()
  {
    return(customerRecs);
  }

  public CustomerRec getCustomerRecord(String ID)
  {
    if (customerRecs == null)
      return(null);
    int n = customerRecs.size();
    CustomerRec crec;
    String cID;
    for (int i = 0; i < n; i++)
    {
      crec = (CustomerRec) customerRecs.get(i);
      cID = crec.getID();
      if (cID.equalsIgnoreCase(ID) == true)
        return(crec);
    }
    return(null);
  }

  // Member variables
  private Vector customerRecs;
  private MetadataRec metadataRecs;
}