
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

import java.lang.String;
import java.io.IOException;
import java.io.FileInputStream;

public class TagValueParser extends Object implements FileParser
{

  public TagValueParser()
  {
    pEngine = new TagValueParserEngine(); // Default tag value parser engine
  }

  TagValueParser(ParserEngine aEngine)
  {
    pEngine = (TagValueParserEngine)aEngine; // Custom engine
  }

  public void processFile(String fileName) throws IOException, Exception,
                                ParserException
  {
    FileInputStream fis;
    byte[] rec;
    int n, r;

    fis = new FileInputStream(fileName);
    try {
      while (true) {
        n = fis.available();
        if (n <= 0)
          break;
        if (n > 10241)
        {
          Exception e = new Exception("File too big.. > 10K");
          throw e;
        }
        if (n > 0)
        {
          // There is data to be read, create a byte array to hold the data
          rec = new byte[n+1];
          r = fis.read(rec);
          if (r == -1)
            break; // break out of the while loop
          try
          {
            pEngine.parseBuffer(rec,n); // This buffer contains all the data
                                        // in the ascii file
          }
          catch (ParserException e)
          {
             throw e;
          }
        }
      }
    } catch (IOException e) { throw e; }
  }

  public String getControlRecord(String tag)
  {
    MetadataControl m;
    m = pEngine.getControlRecord(tag);
    return(m.getValue());
  }

  public String getLayoutRecord(String tag)
  {
    MetadataLayout l;
    l = pEngine.getLayoutRecord(tag);
    return(l.getType());
  }
  // Associated TagValueParserEngine instance
  private TagValueParserEngine pEngine;
}

