
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */

 /*
  * This parser engine transforms text strings into meta data control and
  * layout objects
*/

package com.iconexio.asciiFileAdapter;

import java.util.Hashtable;

public class TagValueParserEngine extends Object implements ParserEngine
{

  public TagValueParserEngine()
  {
    // This is the default state machine for the engine
    psMachine = new TagValueParserStateMachine();
  }

  public void parseBuffer(byte[] buff, int len) throws ParserException
  {
    String startTag;
    String endTag;
    String tagValue;
    String v, d;
    int sFlag = 0;
    int fid = 1;
    int recType; // Control = 0 or Layout = 1

    psMachine.resetParser();
    startTag = null;
    endTag = null;
    tagValue = null;
    recType = 0;

    for (int i = 0; i < len; i++)
    {
      if (sFlag == 1) // Previous error detected do not continue
         break;
      try
      {
        // Call the parser state machine to process the data
        psMachine.processByte(buff[i]);
      } catch(ParserException e) { throw e; }

      if (psMachine.startTagDetected() == true)
      {
        // Start tag has been detected and is stored in the parserEngine
        // Get the tag name stored in the state engine
        startTag = psMachine.getData();
        psMachine.resetDataBuffer();
      } // Start tag detected

      if (psMachine.endTagDetected() == true)
      {
        endTag = psMachine.getData();
        // Validate if the element tags are the same before adding the tag
        // value pair to the Hashtable
        if (startTag.equalsIgnoreCase(endTag) == true)
        {
          if (startTag.equalsIgnoreCase("METADATATYPE") == true)
          {
            // This element defines the meta data type - control or layout
            if (tagValue.equalsIgnoreCase("Layout") == true)
            {
              recType = 1;
              psMachine.resetDataBuffer();
              continue;
            }
            else if (tagValue.equalsIgnoreCase("Control") == true)
            {
              recType = 0;
              psMachine.resetDataBuffer();
              continue;
            }
            else
            {
              recType = -1;
              sFlag = 1;
              ParserException e = new
                        ParserException("Invalid meta data type");
              throw e;
            }
          }
          if (recType == 0)
          {
            // This meta data record has control information
            MetadataControl m = new MetadataControl(fid,startTag,tagValue);
            mRec.addControlRecord(m);
            fid++;
          }
          else if (recType == 1)
          {
            // This meta data record has layout information
            // If there are any domain values extract them from the tagValue
            int j = tagValue.indexOf((int)'|');
            if (j == -1)
            {
              d = new String("NO_DOMAIN_DEFINED");
              MetadataLayout m = new MetadataLayout(fid,startTag,tagValue,d);
              mRec.addLayoutRecord(m);
              fid++;
            }
            else
            {
              d = tagValue.substring(j);
              v = tagValue.substring(0,j);
              MetadataLayout m = new MetadataLayout(fid,startTag,v,d);
              mRec.addLayoutRecord(m);
              fid++;
            }
          }
        }
        else
        {
          // Log data to error log
          sFlag = 1;
          ParserException e = new
                        ParserException("Start Tag " + startTag +
                                        " and End Tag " + endTag +
                                        " do not match");
          throw e;
        } // Start tag and End tag do not match
        psMachine.resetDataBuffer();
      } // End of tag detected
      if (psMachine.tagValueDetected() == true)
      {
        tagValue = psMachine.getData();
        psMachine.resetDataBuffer();
      } // An element value detected
    } // End of for
  } // End of method

  public MetadataControl getControlRecord(String tagName)
  {
    return(mRec.getControlRecord(tagName));
  }

  public MetadataLayout getLayoutRecord(String tagName)
  {
    return(mRec.getLayoutRecord(tagName));
  }
  public void setMetadataRec(MetadataRec aRec)
  {
    this.mRec = aRec;
  }
  // Member variables
  private TagValueParserStateMachine psMachine;
  private MetadataRec mRec;
}
