
/**
 * Title:        ASCII File Resource Adapter<p>
 * Description:  This project is about using existing Java
 * application level classes and encapsulating them
 * in a JCA 1.0 compliant resource adapter<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iConexio.jca.asciiFileRA;

import javax.resource.ResourceException;
import javax.resource.spi.*;

/** This class will return information about the managed connections and the
 * resource managed by it.
 */
public class AFManagedConnectionMetaData implements ManagedConnectionMetaData
{

  public AFManagedConnectionMetaData(ASCIIFileManagedConnection afmc)
  {
    this.afmc = afmc;
  }
  /** This method returns the name of the EIS in this it can be the ASCII
   * file name of the customer data file
   */
  public String getEISProductName() throws ResourceException
  {
    if (mc.isDestroyed()) {
       throw new ResourceException
          ("ManagedConnection has been destroyed");
    }
    // If the managed connection instance stores the customer data file name
    // it can be returned here.
    return(afmc.getASCIIFileName());
  }

  public String getEISProductVersion() throws ResourceException
  {
    /** The version number of the ASCII file cannot be retrieved in most
     * cases
     */
    String versionNo = new String("Version number unknown");
    return(versionNo);
  }

  public int getMaxConnections() throws ResourceException
  {
     // The number of connections to the ASCII file will depend on
     // how many files can be opened by a user or in a session etc.
     // Hence for this example we just return the number 16
     return((int)16);
  }

  public String getUserName() throws ResourceException
  {
    if (mc.isDestroyed()) {
       throw new ResourceException
          ("ManagedConnection has been destroyed");
    }
    // Return the user name stored as part of the credentials
    return(afmc.getPasswordCredentail.getUserName());
  }

  private ASCIIFileManagedConnection afmc;
}




